/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#ifndef TNCUTIL_H_
#define TNCUTIL_H_

#ifdef __WINDOWS__
#define NOMINMAX
#include <windows.h>
#endif

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <stdarg.h>
#include <sstream>
#include "tcg/tnc/tncifimc.h"
#include "tcg/tnc/tncifimv.h"

/**
 * TNCUtil provides several generic helper methods. In addition, it includes
 * an easy to use framework for implementing IMC/IMV pairs.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>09.04.2006 - create class (Daniel Wuttke)</li>
 *   <li>27.02.2008 - added getSymbol (ib)</li>
 *   <li>27.02.2008 - moved getSymbol to seperate file (ib)</li>
 *   <li>06.01.2009 - updated comments (ib)</li>
 * </ul>
 *
 * @namespace TNCUtil
 * @brief TNCUtil provides several generic helper methods.
 * @date 09.04.2006
 * @author Daniel Wuttke
 * @author Ingo Bente (ib)
 */
namespace TNCUtil
{

	/**
	 * Writes at most 'size' bytes from 'format' to 'buf'. 'format' is a
	 * format-string as used by ordinary printf functions.
	 *
	 * @param buf The buffer where to write the array.
	 * @param size Size of the buffer.
	 * @param format A format string.
	 */
    void writeToArray(char *buf, const int size, const char *format, ...);

	/**
	 * Parses the string contained in 'text' according to the specified delimiters.
	 * All retrieved tokens are returned as separate strings in the result vector.
	 *
	 * @param text The text in which tokens are searched.
	 * @param delimiters The delimiters specified as separate chars.
	 */
    std::vector<std::string> * getTokens(const char *text, const char *delimiters);


	/**
	 * Prints the passed wide-character-string with the passed length
	 * on the console.
	 *
	 * @param string A string.
	 * @param lenght The lenght of the string.
	 */
	void printWide(wchar_t *string, int length);


	/**
	 * swaps the two bytes of a 2-byte-number.
	 * <pre>
	 *      +--------+
	 *      |        |
	 * +----|----+---v-----+
	 * | 1. Byte | 2. Byte |
	 * +----^----+---|-----+
	 *      |        |
	 *      +--------+
	 * </pre>
	 *
	 * @param nValue The 2 byte that shall be swapped.
	 *
	 * @return The new, swapped 2 byte.
	 */
	unsigned short byteSwap(unsigned short nValue);


	/**
	 * swaps the four bytes of a 4-byte-number
	 * <pre>
	 *      +----------------------------+
	 *      |        +---------+         |
	 *      |        |         |         |
	 * +----|----+---|-----+---v-----+---v-----+
	 * | 1. Byte | 2. Byte | 3. Byte | 4. Byte |
	 * +----^----+---^-----+---|-----+---|-----+
	 *      |        |         |         |
	 *      |        +---------+         |
	 *      +----------------------------+
	 * </pre>
	 *
	 * @param nLongNumber The 4 byte that shall be swapped.
	 *
	 * @return The new, swapped 4 byte.
	 */
	unsigned long byteSwap4Byte (unsigned long nLongNumber);


	/**
	 * Helper method to create a message type
	 * from vendor ID and message subtype.<br/>
	 * <a href="https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf">
	 * TCG TNC IF-IMC Specification 1.2</a> chapter 3.4.2.5
	 *
	 * @param vendorID The vendor id.<br/>
	 * 		<a href="https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf">
	 * 		TCG TNC IF-IMC Specification 1.2</a> chapter 3.4.2.7
	 * @param subType The message subtype.<br/>
	 * 		<a href="https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf">
	 * 		TCG TNC IF-IMC Specification 1.2</a> chapter 3.4.2.8
	 *
	 * @return Returns the message type.
	 */
	TNC_MessageType formMessageType(TNC_VendorID vendorID, TNC_MessageSubtype subType);


	/**
	 * Helper method to extract the message subtype from a message type.
	 *
	 * @param messageType The message type.
	 *
	 * @return The least significant 8 bits from message type is the message subtype.<br/>
	 * 		   <a href="https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf">
	 * 		   TCG TNC IF-IMC Specification 1.2</a> chapter 3.4.2.5
	 */
	TNC_MessageSubtype parseMessageSubType(TNC_MessageType messageType);


	/**
	 * Helper method to extract the vendor-id from a message type.
	 *
	 * @param messageType The message type.
	 *
	 * @return The most significant 24 bits from message type is the vendor-id.<br/>
	 * 		   <a href="https://www.trustedcomputinggroup.org/specs/TNC/TNC_IFIMC_v1_2_r8.pdf">
	 * 		   TCG TNC IF-IMC Specification 1.2</a> chapter 3.4.2.5
	 */
	TNC_VendorID parseMessageVendorId(TNC_MessageType messageType);

	/**
	 * Generic toString function
	 *
	 * @param t reference of type T
	 *
	 * @return string representation of type t
	 */
	template <class T>
	inline std::string toString (const T& t)
	{
		std::stringstream ss;
		ss << t;
		return ss.str();
	}

	#ifdef __WINDOWS__
		/**
		 * Converts a BSTR string to a std::string.
		 *
		 * @param str The BSTR string.
		 * @return The std::string.
		 */
	    std::string convertWCharString(BSTR str);
	#endif

}

#endif /*TNCUTIL_H_*/
