/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */


#include "IMUnitInfo.h"

#include "../io/TNCLog.h"

#ifndef ABSTRACTIMUNIT_H_
#define ABSTRACTIMUNIT_H_

namespace TNCUtil {

namespace iml {

/**
 * AbstractIMUnit represents the similarities of IMCs and IMVs.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>07.02.2008 - create class (ib)</li>
 *   <li>08.05.2008 - change namespace (mbs)</li>
 * </ul>
 *
 * @class AbstractIMUnit
 * @brief Represents the abstract super class for any concrete IMC/IMV implementation.
 * @date 07.02.2008
 * @author Ingo Bente
 * @author Mike Steinmetz (mbs)
 */
class AbstractIMUnit {

	/*
	 * Friends of AbstractIMUnit (are allowed to call private methods)
	 *
	 * A bug on windows, here we must replace TNC_Result to unsigned long
	 */
	friend unsigned long ::TNC_IMV_NotifyConnectionChange(TNC_IMVID pImvID,
			TNC_ConnectionID connectionID, TNC_ConnectionState newState);
	friend unsigned long ::TNC_IMC_NotifyConnectionChange(TNC_IMCID imcID,
			TNC_ConnectionID connectionID, TNC_ConnectionState newState);
	friend class AbstractIMV;
	friend class AbstractIMC;

public:

	/**
	 * Ctor.
	 *
	 * @param conID The TNC_ConnectionID for the IMC or IMV instance.
	 */
	AbstractIMUnit(TNC_ConnectionID conID);

	/**
	 * Dtor.
	 */
	virtual ~AbstractIMUnit();

	/**
	 * Getter for the current connection state.
	 *
	 * @return The current connection state.
	 */
	TNC_ConnectionState getConnectionState() const;

	/**
	 * Getter for the connection ID.
	 *
	 * @return The connection ID.
	 */
	TNC_ConnectionID getConnectionID() const;

	/**
	 * Hook method for event notification. Is called when the status of the connection changes.
	 */
	virtual TNC_Result notifyConnectionChange();

	/**
	 * Invoked after TNC_IMC_BatchEnding(...) has been called by TNC client/server.
	 * Has to be implemented by a concrete IMC/IMV. All messages received in a batch
	 * have been delivered to the IMC/IMV. Last chance for IMC/IMV to send a message
     * to IMV/IMC
     * .
	 * @param message The message that shall be delivered to the IMC/IMV.
	 * @param messageType The type of the message.
	 * @param messageLength The number of octets in the message.
	 *
	 * @return A result indicating whether the operation was successful or not.
	 */
	virtual TNC_Result batchEnding();

	/**
	 * Invoked after TNC_IMV_ReceiveMessage() has been called by TNC server.
	 * Has to be implemented by a concrete IMV.
	 * This function is called to deliver a message to the respective IMV.
	 *
	 * @param message The message that shall be delivered to the IMV.
	 * @param messageType The type of the message.
	 * @param messageLength The number of octets in the message.
	 *
	 * @return A result indicating whether the operation was succseful or not.
	 */
	virtual TNC_Result receiveMessage(TNC_BufferReference message,
			TNC_UInt32 messageLength, TNC_MessageType messageType);

private:
	virtual TNC_Result notifyConnectionChange(TNC_ConnectionState newState);

	/**
	 * Represents the current connection state. Possible values are defined in tncifimc/v.h
	 * contained in the TNCUtil package.
	 */
	TNC_ConnectionState connectionState;

	/**
	 * Connection ID for this IMUnit object. Is constant for the lifespan of the object.
	 */
	const TNC_ConnectionID connectionID;
};

}  // namespace iml

} // namespace TNCUtil

#endif /*ABSTRACTIMUNIT_H_*/
