/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */


#include "IFIMUnitImpl.h"
#include "hook.h"

#ifdef __WINDOWS__
#  include <windows.h>
#endif // __WINDOWS__

#ifdef WIN32
#define TNC_IMC_EXPORTS
#endif

namespace TNCUtil {

/** @file IFIMCImpl.cpp
 * TNC IF-IMC interface implementation used by TNC server.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>09.05.2006 - create file (Daniel Wuttke)</li>
 *   <li>00.02.2008 - completely redesign (ib & mbs)</li>
 *   <li>08.05.2008 - change namespace (mbs)</li>
 *   <li>10.05.2008 - add some try/catch-blocks and remove some bugs (mbs)</li>
 *   <li>10.05.2008 - add getImcInstances() code (mbs)</li>
 *   <li>08.08.2008 - fixed TNC_IMC_API Bug, added #define TNC_IMC_EXPORTS  (ib)</li>
 * </ul>
 *
 * @date 09.05.2006
 * @author Daniel Wuttke
 * @author Ingo Bente (ib)
 * @author Mike Steinmetz (mbs)
 */

namespace iml {

// Classname for debugging and logging
#define CLASSNAME "IFIMCImpl"

}  // namespace iml

}  // namespace TNCUtil

#ifdef __WINDOWS__

/**
 * Main method for Windows DLL platform binding. Is called by LoadLibrary, FreeLibrary.
 *
 * @param hInstance todo
 * @param fdwReason todo
 * @param pvReserved todo
 * @return todo
 * @todo What does this function do? Check DLL handling on windows.
 */
int WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved) {
    return TRUE;
}
#endif // __WINDOWS__

using namespace TNCUtil::iml;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNC_Result TNC_IMC_Initialize(TNC_IMCID pImcID, TNC_Version minVersion,   *
 *     TNC_Version maxVersion, TNC_Version *pOutActualVersion)               *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_Initialize(TNC_IMCID imcID, TNC_Version minVersion, TNC_Version maxVersion, TNC_Version *pOutActualVersion)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMC_Initialize");

    try {
    	//enter to critical area
    	Semaphore semaphore;

        return TNC_IMUnit_Initialize(imcID, minVersion, maxVersion, TNC_IFIMC_VERSION, pOutActualVersion);

    } catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *      TNC_Result TNC_IMC_NotifyConnectionChange(TNC_IMCID imcID,           *
 *          TNC_ConnectionID connectionID, TNC_ConnectionState newState)     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_NotifyConnectionChange(TNC_IMCID imcID, TNC_ConnectionID connectionID, TNC_ConnectionState newState)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMC_NotifyConnectionChange");

    AbstractIMC * imc = NULL;
    TNC_Result result = TNC_RESULT_SUCCESS;

    try {
    	//enter to critical area
    	Semaphore semaphore;

    	checkPreconditions(imcID);

	    switch (newState) {
			case TNC_CONNECTION_STATE_CREATE:
				{
					/* check imc does not exist */
					bool exist = false;
					try {
						getImcInstances(connectionID);
						exist = true;
					} catch (TNCUtil::exception::ResultException& e) {}
					if (exist)
						throw TNCUtil::exception::ResultException("TNC_IMC_NotifyConnectionChange(TNC_CONNECTION_STATE_CREATE): IMC already exist.", TNC_RESULT_ILLEGAL_OPERATION);

					/* Create concrete IMC instance */
		            log.debug("Create new IMC instance for connection ID 0x%lX.", connectionID);
		            AbstractIMUnit * tmp = createNewIMUnitInstance(connectionID);
		            if (!tmp || !dynamic_cast<AbstractIMC*>(tmp)){
		            	delete tmp;
		            	throw TNCUtil::exception::ResultException("createNewIMUnitInstance(..) returns no AbstractIMC", TNC_RESULT_FATAL);
		            }
		            imcInstances[connectionID] = dynamic_cast<AbstractIMC*>(tmp);
		            imc = getImcInstances(connectionID);
				}
				break;
			case TNC_CONNECTION_STATE_DELETE:
	            // Connection will be deleted. Remove IMC instance from map.
	            log.debug("Delete IMC instance for connection ID 0x%lX.", connectionID);
	            imc = getImcInstances(connectionID);
	            imcInstances.erase(connectionID);
				break;
			case TNC_CONNECTION_STATE_ACCESS_ALLOWED:
			case TNC_CONNECTION_STATE_ACCESS_ISOLATED:
			case TNC_CONNECTION_STATE_ACCESS_NONE:
			case TNC_CONNECTION_STATE_HANDSHAKE:
	            imc = getImcInstances(connectionID);
				break;
			default:
				throw TNCUtil::exception::ResultException("Unknown TNC_ConnectionState!", TNC_RESULT_INVALID_PARAMETER);
		}

	    result = imc->notifyConnectionChange(newState);

	    if (newState == TNC_CONNECTION_STATE_DELETE)
	            delete imc;

	    return result;

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        if (newState == TNC_CONNECTION_STATE_DELETE)
                delete imc;
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        if (newState == TNC_CONNECTION_STATE_DELETE)
                delete imc;
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
        if (newState == TNC_CONNECTION_STATE_DELETE)
                delete imc;
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        if (newState == TNC_CONNECTION_STATE_DELETE)
                delete imc;
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *           TNC_Result TNC_IMC_BeginHandshake(TNC_IMCID imcID,              *
 *               TNC_ConnectionID connectionID)                              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_BeginHandshake(TNC_IMCID imcID, TNC_ConnectionID connectionID)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMC_BeginHandshake");

    try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkPreconditions(imcID);

	    return getImcInstances(connectionID)->beginHandshake();

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *   TNC_Result TNC_IMC_ReceiveMessage(TNC_IMCID imcID,                      *
 *       TNC_ConnectionID connectionID, TNC_BufferReference messageBuffer,   *
 *       TNC_UInt32 messageLength, TNC_MessageType messageType)              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_ReceiveMessage(TNC_IMCID imcID, TNC_ConnectionID connectionID, TNC_BufferReference messageBuffer, TNC_UInt32 messageLength, TNC_MessageType messageType)
{
	TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMC_ReceiveMessage");

    try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkReciveMessageParameter(imcID, messageBuffer, messageLength, messageType);

	    return getImcInstances(connectionID)->receiveMessage(messageBuffer, messageLength, messageType);

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *             TNC_Result TNC_IMC_BatchEnding(TNC_IMCID imcID,               *
 *                 TNC_ConnectionID connectionID)                            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_BatchEnding(TNC_IMCID imcID, TNC_ConnectionID connectionID)
{
	TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMC_BatchEnding");

	try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkPreconditions(imcID);

	    return getImcInstances(connectionID)->batchEnding();

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
    return TNC_RESULT_SUCCESS;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *               TNC_Result TNC_IMC_Terminate(TNC_IMCID imcID)               *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_Terminate(TNC_IMCID imcID) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMC_Terminate");

	try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkPreconditions(imcID);

	    std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMC *>::iterator iter = imcInstances.begin();
	    std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMC *>::iterator end  = imcInstances.end();

        // Delete all concrete IMC instances
	    while (iter!=end) {
		    AbstractIMC * imc = iter->second;
	    	iter++;
	    	delete imc;
	    }

	    // Erase map
	    imcInstances.clear();

    	// remove the tnccFunctionPointers
    	tnccFunctionPointers.bindFunctionPointer          = NULL;
    	tnccFunctionPointers.reportMessageTypesPointer    = NULL;
    	tnccFunctionPointers.requestHandshakeRetryPointer = NULL;
    	tnccFunctionPointers.sendMessagePointer           = NULL;

    	actualVersion = 0;
        imUnitID = 0;
	    initialized = false;

	    return TNC_RESULT_SUCCESS;

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNC_Result TNC_IMC_ProvideBindFunction(TNC_IMCID imcID,                   *
 *     TNC_TNCC_BindFunctionPointer bindFunction)                            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_ProvideBindFunction(TNC_IMCID imcID, TNC_TNCC_BindFunctionPointer bindFunction) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMC_ProvideBindFunction");

    try {
    	//enter to critical area
    	Semaphore semaphore;

    	// check preconditions
	    checkPreconditions(imcID);

	    // check parameter
        if (bindFunction != NULL) {
        	// set the tnccFunctionPointers
        	tnccFunctionPointers.bindFunctionPointer = bindFunction;
        	tnccFunctionPointers.reportMessageTypesPointer = (TNC_TNCC_ReportMessageTypesPointer) getTNCFunctionPointer("TNC_TNCC_ReportMessageTypes", bindFunction);
        	tnccFunctionPointers.requestHandshakeRetryPointer = (TNC_TNCC_RequestHandshakeRetryPointer) getTNCFunctionPointer("TNC_TNCC_RequestHandshakeRetry", bindFunction);
        	tnccFunctionPointers.sendMessagePointer= (TNC_TNCC_SendMessagePointer) getTNCFunctionPointer("TNC_TNCC_SendMessage", bindFunction);

            // Report messages types to TNC client
        	IMUnitInfo imUnitInfo = getIMUnitInfo();
        	tnccFunctionPointers.reportMessageTypesPointer(imUnitID, imUnitInfo.supportedTypes, imUnitInfo.typeCount);
        	if (log.isDebug())
        		for (unsigned int i = 0; i < imUnitInfo.typeCount; ++i)
					log.debug("Supported Type: %8lX", imUnitInfo.supportedTypes[i]);
        }
	    log.debug("ProvideBindFunction succesful: tnccFunctionPointers has been initialized.");

        // Everything is OK
        return TNC_RESULT_SUCCESS;

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
	    return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
	    return TNC_RESULT_FATAL;
    }
}
