/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#include "IFIMUnitImpl.h"
#include "../TNCUtil.h"
#include <sstream>

namespace TNCUtil {

namespace iml {

// Classname for logging
#define CLASSNAME "IFIMUnitImpl"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                     TNC_Result TNC_IMUnit_Initialize()                    *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMUnit_Initialize(TNC_IMUnitID id, TNC_Version minVersion, TNC_Version maxVersion, TNC_Version ifIMUnitVersion, TNC_Version *pOutActualVersion) throw (TNCUtil::exception::ResultException)
{
	TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMUnit_Initialize");
    // Check, if TNC_IMUnit_Initialize() has already been invoked
    if (initialized)
    	throw TNCUtil::exception::ResultException("TNC_IMUnit_Initialize() has already been invoked!", TNC_RESULT_ALREADY_INITIALIZED);

    // Check API versions
	actualVersion = getVersion(minVersion, maxVersion, ifIMUnitVersion, pOutActualVersion);

    *pOutActualVersion = actualVersion;
    imUnitID = id;
    initialized = true;

    log.debug("Initialize complete");

    // Everything is OK
    return TNC_RESULT_SUCCESS;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                      void checkIMUnitInitialized()                        *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void checkIMUnitInitialized() throw (TNCUtil::exception::ResultException)
{
    if (!initialized)
    	throw TNCUtil::exception::ResultException("TNC_IMC/IMV_Initialize() has to be invoked first!", TNC_RESULT_NOT_INITIALIZED);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *    std::string operator+(const std::string & s, const TNC_UInt32 & i)     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
std::string operator+(const std::string & s, const TNC_UInt32 & i)
{
	std::ostringstream temp;
	temp << s << i;
	return temp.str();
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNC_Version getVersion(TNC_Version min, TNC_Version max,                  *
 *     TNC_Version support, TNC_Version *pOutActualVersion)                  *
 *     throw (TNCUtil::exception::ResultException)                                          *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

TNC_Version getVersion(TNC_Version min, TNC_Version max, TNC_Version support, TNC_Version *pOutActualVersion) throw (TNCUtil::exception::ResultException)
{
    if (max < min)
        throw TNCUtil::exception::ResultException(std::string("MaxVersion ") + max + " lower than minVersion " + min + "!", TNC_RESULT_INVALID_PARAMETER);
    else if (support < min || max < support)
        throw TNCUtil::exception::ResultException(std::string("This IMUnit does not support API versions between ") + min + " and " + max, TNC_RESULT_NO_COMMON_VERSION);
    else if (pOutActualVersion == NULL)
        throw TNCUtil::exception::ResultException("The output parameter pOutActualVersion is not valid!", TNC_RESULT_INVALID_PARAMETER);
    return support;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                       void checkPreconditions()                           *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void checkPreconditions(TNC_IMUnitID id) throw (TNCUtil::exception::ResultException)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "checkPreconditions");

    checkIMUnitInitialized();

    if(imUnitID != id){
    	throw TNCUtil::exception::ResultException(std::string("The IMUnitIDs are unequal: ") + imUnitID + " != " + id, TNC_RESULT_INVALID_PARAMETER);
    }

    log.debug("Preconditions ok for %d", id);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                     void checkReciveMessageParameter()                    *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void checkReciveMessageParameter(TNC_IMUnitID id, TNC_BufferReference messageBuffer, TNC_UInt32 messageLength, TNC_MessageType messageType) throw (TNCUtil::exception::ResultException)
{
    checkPreconditions(id);

    if (messageLength == 0 && messageBuffer != NULL)
    	throw TNCUtil::exception::ResultException("Error: TNCS or TNCC is not specification conform: messageLength == 0 && messageBuffer != NULL", TNC_RESULT_INVALID_PARAMETER);
    else if (messageLength != 0 && messageBuffer == NULL)
    	throw TNCUtil::exception::ResultException("Error: messageLength != 0 && messageBuffer == NULL", TNC_RESULT_INVALID_PARAMETER);

    if (parseMessageSubType(messageType) == 0xff)
    	throw TNCUtil::exception::ResultException("Error: TNCS or TNCC is not specification conform: MessageSubType == 0xff", TNC_RESULT_INVALID_PARAMETER);
    if (parseMessageVendorId(messageType) == 0xffffff)
    	throw TNCUtil::exception::ResultException("Error: TNCS or TNCC is not specification conform: VendorID == 0xffffff", TNC_RESULT_INVALID_PARAMETER);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *     void * getTNCFunctionPointer(char *functionName,                      *
 *            TNC_TNC_BindFunctionPointer tncBindFunction)                   *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void * getTNCFunctionPointer(const char *functionName, TNC_BindFunctionPointer tncBindFunction) throw (TNCUtil::exception::ResultException)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "getTNCFunctionPointer");

    void *tncFunctionPointer = NULL;
    tncBindFunction(imUnitID, const_cast<char*>(functionName), &tncFunctionPointer);
    if (tncFunctionPointer == NULL) {
        throw TNCUtil::exception::ResultException(std::string("TNC client/server function was not found: ") + functionName, TNC_RESULT_FATAL);
    } else {
        log.debug("TNC server function %s was found.", functionName);
    }
    return tncFunctionPointer;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *       TNCUtil::iml::AbstractIMV * getImvInstances(TNC_ConnectionID)       *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCUtil::iml::AbstractIMV * getImvInstances(TNC_ConnectionID id) throw (TNCUtil::exception::ResultException)
{
	std::map<TNC_ConnectionID, AbstractIMV *>::iterator result = imvInstances.find(id);
	if (result == imvInstances.end())
    	throw TNCUtil::exception::ResultException(TNC_RESULT_INVALID_PARAMETER, "Connection ID 0x%lX is not valid!", id);

	return imvInstances[id];
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *       TNCUtil::iml::AbstractIMC * getImcInstances(TNC_ConnectionID)       *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCUtil::iml::AbstractIMC * getImcInstances(TNC_ConnectionID id) throw (TNCUtil::exception::ResultException)
{
	std::map<TNC_ConnectionID, AbstractIMC *>::iterator result = imcInstances.find(id);
	if (result == imcInstances.end())
    	throw TNCUtil::exception::ResultException(TNC_RESULT_INVALID_PARAMETER, "Connection ID 0x%lX is not valid!", id);

	return imcInstances[id];
}


TNC_IMUnitID imUnitID;

struct TNCC_FunctionPointers tnccFunctionPointers;

struct TNCS_FunctionPointers tncsFunctionPointers;


std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMV *> imvInstances;

std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMC *> imcInstances;

TNC_Version actualVersion;


bool initialized = false;


bool isSemaphoreInitialized = false;

/**
 * Semaphore object for synchronisation
 *
 * @see SemaphoreInit
 */
sem_t semaphoreObject;

/**
 * Semaphore class for initialize the semaphoreObject. Initialize and destroy
 * the semaphore automatically.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>08.02.2008 - create class (mbs)</li>
 * </ul>
 *
 * @class SemaphoreInit
 * @brief Semaphore class for initialize the semaphoreObject
 * @date 08.02.2008
 * @author Mike Steinmetz (mbs)
 */

class SemaphoreInit {
public:
	SemaphoreInit()
	{
//		FIXME - the log produce here a segmentaition fault
//		TNCLog &log = TNCLog::getLog("SemaphoreInit", "Constructor");
//	    log.debug("Initialize semaphore.");
	    int pshared = 1;
	    int value = 1;

		if (sem_init(&semaphoreObject, pshared, value)) {
			std::cerr << "Error on initialize semaphore" << std::endl;
			throw TNCUtil::exception::ResultException("Error by initialize Semaphore", TNC_RESULT_FATAL);
		}

		isSemaphoreInitialized = true;
//	    log.debug("Finished initialize semaphore.");
	}

	~SemaphoreInit()
	{
		//sem_close return always error
//		if (sem_close(&semaphoreObject))
//			std::cerr << "Error on close semaphore" << std::endl;
	}
}semaphoreInit_instance;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                          Semaphore::Semaphore()                           *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Semaphore::Semaphore() throw (TNCUtil::exception::ResultException)
{
    TNCLog &log = TNCLog::getLog("Semaphore", "Constructor");

	if (!isSemaphoreInitialized)
		throw TNCUtil::exception::ResultException("Very FATAL ERROR. Semaphore isn't initialize.", TNC_RESULT_FATAL);

	if (sem_wait(&semaphoreObject))
		throw TNCUtil::exception::ResultException("Error by synchronized Semaphore. (in wait)", TNC_RESULT_FATAL);

	log.info("Begin critical area.");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                          Semaphore::~Semaphore()                          *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
Semaphore::~Semaphore()
{
    TNCLog &log = TNCLog::getLog("Semaphore", "Destructor");

	if (sem_post(&semaphoreObject))
		throw TNCUtil::exception::ResultException("Error by synchronized Semaphore. (in post)", TNC_RESULT_FATAL);

	log.info("Leave critical area.");
}

}  // namespace iml

}  // namespace TNCUtil
