/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#include "IFIMUnitImpl.h"
#include "hook.h"

#ifdef __WINDOWS__
#  include <windows.h>
#endif // __WINDOWS__

namespace TNCUtil {

/**
 * TNC IF-IMV interface implementation used by TNC server.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>09.05.2006 - create class (Daniel Wuttke)</li>
 *   <li>00.02.2008 - completely redesign (ib & mbs)</li>
 *   <li>12.02.2008 - attributeFunctions set to NULL in terminate (ib)</li>
 *   <li>08.05.2008 - change namespace (mbs)</li>
 *   <li>10.05.2008 - add some try/catch-blocks and remove some bugs (mbs)</li>
 *   <li>10.05.2008 - add getImvInstances() code (mbs)</li>
 * </ul>
 *
 * @date 09.05.2006
 * @author Daniel Wuttke
 * @author Ingo Bente (ib)
 * @author Mike Steinmetz
 *
 */
namespace iml {

// Classname for debugging and logging
#define CLASSNAME "IFIMVImpl"

#ifdef WIN32
#define TNC_IMV_EXPORTS
#endif

}  // namespace iml

}  // namespace TNCUtil

using namespace TNCUtil::iml;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNC_Result TNC_IMV_Initialize(TNC_IMVID imvID, TNC_Version minVersion,    *
 *     TNC_Version maxVersion, TNC_Version *pOutActualVersion)               *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_Initialize(TNC_IMVID imvID, TNC_Version minVersion, TNC_Version maxVersion, TNC_Version *pOutActualVersion)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMV_Initialize");

    try {
    	//enter to critical area
    	Semaphore semaphore;

    	return TNC_IMUnit_Initialize(imvID, minVersion, maxVersion, TNC_IFIMV_VERSION, pOutActualVersion);

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *      TNC_Result TNC_IMV_NotifyConnectionChange(TNC_IMVID pImvID,          *
 *          TNC_ConnectionID connectionID, TNC_ConnectionState newState)     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_NotifyConnectionChange(TNC_IMVID imvID, TNC_ConnectionID connectionID, TNC_ConnectionState newState)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMV_NotifyConnectionChange");

    AbstractIMV * imv = NULL;
    TNC_Result result = TNC_RESULT_SUCCESS;

    try {
    	//enter to critical area
    	Semaphore semaphore;

    	checkPreconditions(imvID);

	    switch (newState) {
			case TNC_CONNECTION_STATE_CREATE:
				{
					/* check that imv does not exist */
					bool exist = false;
					try {
						getImvInstances(connectionID); // thorws exception if imv found
						exist = true;
					} catch (TNCUtil::exception::ResultException& e) {}
					if (exist)
						throw TNCUtil::exception::ResultException("TNC_IMV_NotifyConnectionChange(TNC_CONNECTION_STATE_CREATE): IMV already exist.", TNC_RESULT_ILLEGAL_OPERATION);

					/* Create concrete IMV instance */
		            log.debug("Create new IMV instance for connection ID 0x%lX.", connectionID);
		            AbstractIMUnit * tmp = createNewIMUnitInstance(connectionID);
		            if (!tmp || !dynamic_cast<AbstractIMV*>(tmp)){
		            	delete tmp;
		            	throw TNCUtil::exception::ResultException("createNewIMUnitInstance(..) returns no AbstractIMV", TNC_RESULT_FATAL);
		            }
		            imvInstances[connectionID] = dynamic_cast<AbstractIMV*>(tmp);
		            imv = getImvInstances(connectionID);
				}
				break;
			case TNC_CONNECTION_STATE_DELETE:
	            // Connection will be deleted. Remove IMV instance from map.
	            log.debug("Delete IMV instance for connection ID 0x%lX.", connectionID);
	            imv = getImvInstances(connectionID);
	            imvInstances.erase(connectionID);
				break;
			case TNC_CONNECTION_STATE_ACCESS_ALLOWED:
			case TNC_CONNECTION_STATE_ACCESS_ISOLATED:
			case TNC_CONNECTION_STATE_ACCESS_NONE:
			case TNC_CONNECTION_STATE_HANDSHAKE:
	            imv = getImvInstances(connectionID);
				break;
			default:
				throw TNCUtil::exception::ResultException("Unknown TNC_ConnectionState!", TNC_RESULT_INVALID_PARAMETER);
		}

	    result = imv->notifyConnectionChange(newState);

	    if (newState == TNC_CONNECTION_STATE_DELETE)
	            delete imv;

	    return result;

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        if (newState == TNC_CONNECTION_STATE_DELETE)
                delete imv;
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        if (newState == TNC_CONNECTION_STATE_DELETE)
                delete imv;
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    if (newState == TNC_CONNECTION_STATE_DELETE)
	            delete imv;
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        if (newState == TNC_CONNECTION_STATE_DELETE)
                delete imv;
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *   TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID pImvID,                     *
 *       TNC_ConnectionID connectionID, TNC_BufferReference messageBuffer,   *
 *       TNC_UInt32 messageLength, TNC_MessageType messageType)              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID imvID, TNC_ConnectionID connectionID, TNC_BufferReference messageBuffer, TNC_UInt32 messageLength, TNC_MessageType messageType)
{
	TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMV_ReceiveMessage");

    try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkReciveMessageParameter(imvID, messageBuffer, messageLength, messageType);

	    return getImvInstances(connectionID)->receiveMessage(messageBuffer, messageLength, messageType);

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *        TNC_Result TNC_IMV_SolicitRecommendation(TNC_IMVID imvID,          *
 *            TNC_ConnectionID connectionID)                                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_SolicitRecommendation(TNC_IMVID imvID, TNC_ConnectionID connectionID)
{
	TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMV_SolicitRecommendation");

    try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkPreconditions(imvID);

	    return getImvInstances(connectionID)->solicitRecommendation();

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *           TNC_Result TNC_IMV_BatchEnding(TNC_IMVID pImvID,                *
 *               TNC_ConnectionID connectionID)                              *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

TNC_Result TNC_IMV_BatchEnding(TNC_IMVID imvID, TNC_ConnectionID connectionID)
{
	TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMV_BatchEnding");

	try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkPreconditions(imvID);

	    return getImvInstances(connectionID)->batchEnding();

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
    return TNC_RESULT_SUCCESS;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *               TNC_Result TNC_IMV_Terminate(TNC_IMVID imvID)               *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_Terminate(TNC_IMVID imvID)
{
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMV_Terminate");

	try {
    	//enter to critical area
    	Semaphore semaphore;

	    checkPreconditions(imvID);

	    std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMV *>::iterator iter = imvInstances.begin();
	    std::map<TNC_ConnectionID, TNCUtil::iml::AbstractIMV *>::iterator end  = imvInstances.end();

        // Delete all concrete IMC instances
	    while (iter!=end) {
		    AbstractIMV * imv = iter->second;
	    	iter++;
	    	delete imv;
	    }

	    // Erase map
	    imvInstances.clear();

    	// remove the tncsFunctionPointers
    	tncsFunctionPointers.bindFunctionPointer          = NULL;
    	tncsFunctionPointers.provideRecommendationPointer = NULL;
    	tncsFunctionPointers.reportMessageTypesPointer    = NULL;
    	tncsFunctionPointers.requestHandshakeRetryPointer = NULL;
    	tncsFunctionPointers.sendMessagePointer           = NULL;

    	// remove optional TNCS function pointers
    	tncsFunctionPointers.getAttributePointer          = NULL;
    	tncsFunctionPointers.setAttributePointer          = NULL;

    	actualVersion = 0;
        imUnitID = 0;
	    initialized = false;

	    return TNC_RESULT_SUCCESS;

	} catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * TNC_Result TNC_IMV_ProvideBindFunction(TNC_IMVID pImvID,                  *
 *     TNC_TNCS_BindFunctionPointer bindFunction)                            *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_ProvideBindFunction(TNC_IMVID imvID, TNC_TNCS_BindFunctionPointer bindFunction) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "TNC_IMV_ProvideBindFunction");

    try {
    	//enter to critical area
    	Semaphore semaphore;

	    // check preconditions
	    checkPreconditions(imvID);

	    // check parameter
        if (bindFunction != NULL) {
        	// set the tncsFunctionPointers
        	tncsFunctionPointers.bindFunctionPointer = bindFunction;
        	tncsFunctionPointers.provideRecommendationPointer = (TNC_TNCS_ProvideRecommendationPointer) getTNCFunctionPointer("TNC_TNCS_ProvideRecommendation", bindFunction);
        	tncsFunctionPointers.reportMessageTypesPointer = (TNC_TNCS_ReportMessageTypesPointer) getTNCFunctionPointer("TNC_TNCS_ReportMessageTypes", bindFunction);
        	tncsFunctionPointers.requestHandshakeRetryPointer = (TNC_TNCS_RequestHandshakeRetryPointer) getTNCFunctionPointer("TNC_TNCS_RequestHandshakeRetry", bindFunction);
        	tncsFunctionPointers.sendMessagePointer = (TNC_TNCS_SendMessagePointer) getTNCFunctionPointer("TNC_TNCS_SendMessage", bindFunction);
        	// OPTIONAL TNCS functions
        	try {
            	tncsFunctionPointers.getAttributePointer = (TNC_TNCS_GetAttributePointer) getTNCFunctionPointer("TNC_TNCS_GetAttribute", bindFunction);
			} catch (TNCUtil::exception::ResultException e) {
				tncsFunctionPointers.getAttributePointer = NULL;
			}
			try {
            	tncsFunctionPointers.setAttributePointer = (TNC_TNCS_SetAttributePointer) getTNCFunctionPointer("TNC_TNCS_SetAttribute", bindFunction);
			} catch (TNCUtil::exception::ResultException e) {
				tncsFunctionPointers.setAttributePointer = NULL;
			}

            // Report messages types to TNC server
        	IMUnitInfo imUnitInfo = getIMUnitInfo();
        	tncsFunctionPointers.reportMessageTypesPointer(imUnitID, imUnitInfo.supportedTypes, imUnitInfo.typeCount);
        	if (log.isDebug())
        		for (unsigned int i = 0; i < imUnitInfo.typeCount; ++i)
					log.debug("Supported Type: %8lX", imUnitInfo.supportedTypes[i]);
        }

	    log.debug("ProvideBindFunction succesful: tncsFunctionPointers has been .");

        // Everything is OK
        return TNC_RESULT_SUCCESS;

    } catch (TNCUtil::exception::ResultException& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return e.getReturnValue();
	} catch (TNCUtil::exception::Exception& e) {
		// Oh shit, we detect error!
        log.error(e.getMessage());
        return TNC_RESULT_FATAL;
	} catch (std::exception &e) {
		// Oh shit, we detect an std error!
	    log.error("std::exception: %s", e.what());
	    return TNC_RESULT_FATAL;
    } catch (...) {
		// Detect unkwnown error!
        log.error("UNKOWN ERROR");
        return TNC_RESULT_FATAL;
    }
}
