/*
 * This software is Copyright (C) 2006-2008
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact tnc@inform.fh-hannover.de for details.
 *
 * http://tnc.inform.fh-hannover.de/
 */

#ifndef TNCIOHELPER_H_
#define TNCIOHELPER_H_

#include <iostream>
#include <fstream>
#include <string.h>
#include <vector>
#include <sys/types.h>
#include "../exception/Exception.h"

namespace TNCUtil {

/** @file
 *
 * Can Print Binary-Data.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>13.06.2006 - create class (Daniel Wuttke)</li>
 *   <li>00.02.2008 - refactoring (ib & mbs)</li>
 *   <li>10.05.2008 - add function readAllProperties (mbs)</li>
 *   <li>10.05.2008 - change some std::ifstram in std::istram (mbs)</li>
 * </ul>
 *
 * @author Daniel Wuttke
 * @author Ingo Bente
 * @author Mike Steinmetz (mbs)
 *
 * @date 13.06.2006
 */
namespace io {

	/**
	 * Print binary data on std::ostream
	 *
	 * @param packet the binary data
	 * @param packetLength the length of binary data
	 * @param text a description of the binary data (default is "")
	 * @param cout the ostream for print the binary data (default is std::cout)
	 */
	void printBinaryOut(const unsigned char * const packet, unsigned long packetLength, std::string text = "", std::ostream &cout = std::cout);

	/**
	 * Print binary data on std::ostream
	 *
	 * @param packet the binary data
	 * @param packetLength the length of binary data
	 * @param text a description of the binary data (default is "")
	 * @param cout the ostream for print the binary data (default is std::cout)
	 */
	void printBinaryOut(const char * const packet, unsigned long packetLength, std::string text = "", std::ostream &cout = std::cout);

    /**
     * Opens the file with the specified filename. After work is done
     * closeFile(in) has to be called.
     *
     * @param filename The name of the file that shall be opened.
     * @param errOut if true and error in open file then print error on std::cerr, otherwise none
     *
     * @return Pointer to the ifstream for the opened file.
     */
    std::ifstream * openFile(std::string filename, bool errOut = true);

    /**
     * Close the file and release resources.
     *
     * @param in Stream of the file that shall ce closed.
     */
    void closeFile(std::ifstream *in);

    /**
     * Reads one line from file ans writes it into the buffer
     * including '\0' and excluding '\n'.
     *
     * @param in Stream of the file from which shall be read.
     * @param buf Buffer where the read line is saved.
     * @param size Size of the buffer.
     */
    bool readLine(std::istream *in, char *buf, const int size);

    /**
     * Reads a property value from the file and writes it into the buffer.
     *
     * @param in Stream of the file from which shall be read.
     * @param buf Buffer where the read bytes are written to.
     * @param size Size of the buffer.
     * @param propertyName Name of the property that shall be read.
     */
    bool readProperty(std::ifstream *in, char *buf, const int size, const char *propertyName);

    /**
     * Reads all properties from the file.
     *
     * @param in Stream of the file from which shall be read.
     * @return A vector of key value pairs.
     */
	std::vector<std::pair<std::string,std::string> > readAllProperties(std::istream *in);

	/**
     * Reads characters from the file until delimiter is found and
     * writes them into the buffer.
     *
     * @param in Stream of the file from which shall be read.
     * @param buf Buffer where the read bytes are written to.
     * @param size Size of the buffer.
     * @param delimiter The reading process stops when a delimter is found.
     */
    bool read(std::istream *in, char *buf, const int size, char delimiter);

    /**
     * Opens a file to write. After work is done closeFile(out) has
     * has to be called.
     *
     * @param filename The name of the file.
     * @retuen A pointer to the ofstream for the file.
     */
    std::ofstream * openOutputFile(std::string filename);

    /**
     * Close the file and release resources.
     *
     * @param out Pointer to the ofstream that shall be closed.
     */
    void closeFile(std::ofstream *out);

    /**
     * Writes characters to the file.
     *
     * @param out Pointer to the ofstream of the file.
     * @param buf Buffer which holds the characters that shall be written to the file.
     * @param size Size of the buffer.
     */
    void write(std::ofstream *out, const char *buf, const int size);

    /**
     * Trims the string according to the specified delimiters.
     *
     * @param source The string that shall be trimmed.
     * @param delims Delimiters specified as separate char's.
     * @return The greatest substring from source that does not contain any character
     *         specified as delimiter.
     */
    std::string trim(std::string const& source, char const* delims = " \t\r\n");

    /**
	 * Parses the file (default is /etc/tnc_config). Processes only lines starting with 'IMV ' and ignores any other line.
	 * Caller must delete the return value.
	 *
	 * @param file (default /etc/tnc_config)
	 *
	 * @return A pointer to a vector containing strings. Every two strings form a pair as follows:
	 *         <imvName, imvSO>.
	 */
    std::vector<std::string> *parseTncConfigFile(const char * file = "/etc/tnc_config") throw (TNCUtil::exception::Exception);
}  // namespace io

}  // namespace TNCUtil

#endif /*TNCIOHELPER_H_*/
