/*
 * This software is Copyright (C) 2006 FH Hannover
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to share the
 * source code of your application with everyone you distribute it to, and you
 * also want to give them the right to share who uses it. If you wish to use
 * TNC@FHH under Open Source Licensing, you must contribute all your source code
 * to the open source community in accordance with the GPL Version 2 when your
 * application is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are creating
 * proprietary applications and you are not prepared to distribute and share the
 * source code of your application. Contact tnc@inform.fh-hannover.de for details.
 */

/**
 * @class TNCLog
 * @brief This class provides arbitrary logging functions.
 * 
 * Header file for TNCLog. Provides a basic logging mechanism. For each class/method-pair one instance
 * of TNCLog class is created.
 * 
 * The following loglevels are supported: error < warn < info < debug.
 *
 * @author: Daniel Wuttke
 * @date: 09/04/2006
 */
#ifndef TNCLOG_H_
#define TNCLOG_H_

#include <map>
#include <iostream>
#include <fstream>
#include <string>
#include <stdarg.h>
#include "TNCIOHelper.h"

// Debug levels
#define TNC_LOG_LEVEL_ERROR 0
#define TNC_LOG_LEVEL_WARN  1
#define TNC_LOG_LEVEL_INFO  2
#define TNC_LOG_LEVEL_DEBUG 3

// Default log level
#define TNC_DEFAULT_LOG_LEVEL TNC_LOG_LEVEL_WARN

// Default log buffer size
#define LOG_BUFFER_SIZE_DEFAULT 65536

class TNCLog
{
    private:
        /**
         * Map with TNCLog instances
         */
        static std::map<std::string, TNCLog*> logInstances;

        /** 
         * Output stream for debug, info and warn messages (default: cout)
         */
        static std::ostream *out;

        /** 
         * Output stream for error messages (default: cerr)
         */
        static std::ostream *err;

        /** 
         * Buffer size for log messages (default: 65536)
         */
        static int logBufferSize;

        /**
         * Classname of TNCLog instance
         */
        const char *className;
        
        /**
         * Methodname of TNCLog instance 
         */
        const char *methodName;
        
        /**
         * Debug level of TNCLog instance
         */
        int debugLevel;

        /**
         * Private constructor. Use static helper method for instantiation.
         * 
         * @param className The classname used as first part for identifying the caller.
         * @param methodName The methodname used as second part for identifying the caller.
         * @param propertyFile Name of the file where Logging parameters are specified.
         * 
         */
        TNCLog(const char *propertyFile, const char *className, const char *methodName);

        /**
         * Dtor.
         */
        virtual ~TNCLog();

    public:
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= debug.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
    	void debug(const std::string &message);
        
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= info.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
        void info(const std::string &message);
        
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= warn.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
        void warn(const std::string &message);
        
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= error.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
        void error(const std::string &message);
        
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= debug.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
        void debug(const char *message, ...);
        
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= info.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
        void info(const char *message, ...);
        
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= warn.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
        void warn(const char *message, ...);
        
    	/**
    	 * Writes the message to the logfile if the current loglevel is
    	 * >= error.
    	 * 
    	 * @param message The message that shall be logged.
    	 */
        void error(const char *message, ...);

		/**
		 * Checks if the current loglevel ist >= debug.
		 */
        bool isDebug();
        
		/**
		 * Checks if the current loglevel ist >= info.
		 */
        bool isInfo();
        
		/**
		 * Checks if the current loglevel ist >= warn.
		 */
        bool isWarn();
        
		/**
		 * Checks if the current loglevel ist >= error.
		 */
        bool isError();

        /**
         * Helper method to create a new TNCLog instance for a class/method-pair. If no
         * property file name is specified the default value will be used.
         * 
         * @param className The classname used as first part for identifying the caller.
         * @param methodName The methodname used as second part for identifying the caller.
         * @param propertyFile Name of the file where Logging parameters are specified.
         */
        static TNCLog & getLog(const char *className, const char *methodName, const char *propertyFile = NULL);

        /**
         * Helper method to delete all TNCLog instances.
         */
        static void deleteLogs();

        /**
         * Sets the output stream for debug, info and warn messages.
         * 
         * @param out The outputsream that shall be used.
         */
        static void setOutputStream(std::ostream &out);

        /**
         * Sets the output stream for error messages.
         * 
         * @param err The outputsream that shall be used.
         */
        static void setErrOutputStream(std::ostream &err);

        /**
         * Sets the buffer size for log messages.
         * 
         * @param bufferSize The new size of the buffer.
         */
        static void setLogBufferSize(int bufferSize);
};

#endif /*TNCLOG_H_*/
