#ifdef __WINDOWS__
/*
 * This software is Copyright (C) 2006 FH Hannover
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to share the
 * source code of your application with everyone you distribute it to, and you
 * also want to give them the right to share who uses it. If you wish to use
 * TNC@FHH under Open Source Licensing, you must contribute all your source code
 * to the open source community in accordance with the GPL Version 2 when your
 * application is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are creating
 * proprietary applications and you are not prepared to distribute and share the
 * source code of your application. Contact tnc@inform.fh-hannover.de for details.
 */

/**
 * TNCRegistryHelper implementation. Provides helper methods to access
 * the Windows Registry.
 * 
 * @author: Daniel Wuttke
 *         Mike Steinmetz (mbs)
 * @date: 18/05/2006 
 */

#include "TNCRegistryHelper.h"
#include "../io/TNCLog.h"

// Classname for debugging and logging
#define CLASSNAME "TNCRegistryHelper"

char * TNCRegistryHelper::getRegString(HKEY preservedKey, const char *subKey, const char *valueName, char *defaultValue) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "getRegString");
    
    HKEY key;
    LPBYTE data;
    DWORD size;
	long result;
	
    if ((result=RegOpenKeyEx(preservedKey, subKey, 0, KEY_READ, &key)) != ERROR_SUCCESS) {
   		log.info("RegOpenKeyEx() %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return defaultValue;
    }
    
    if ((result=RegQueryValueEx(key, valueName, 0,  NULL, NULL, &size)) != ERROR_SUCCESS) {
        RegCloseKey(key);
   		log.info("RegQueryValueEx() [1] %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return defaultValue;
    }
    data = (BYTE*) calloc(sizeof(BYTE), size);

    if ((result=RegQueryValueEx(key, valueName, 0, NULL, data, &size)) != ERROR_SUCCESS) {
        RegCloseKey(key);
   		log.info("RegQueryValueEx() [2] %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return defaultValue;
    }
    RegCloseKey(key);

    return (char*) data;
}

DWORD TNCRegistryHelper::getRegNumber(HKEY preservedKey,const char *subKey, const char *valueName, DWORD defaultValue) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "getRegNumber");

    HKEY key;
    BYTE data = 0;
    DWORD size;
    long result;

    if ((result=RegOpenKeyEx(preservedKey, subKey, 0, KEY_READ, &key)) != ERROR_SUCCESS) {
   		log.info("RegOpenKeyEx() %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return defaultValue;
    }
    
    if ((result=RegQueryValueEx(key, valueName, 0,  NULL, NULL, &size)) != ERROR_SUCCESS) {
        RegCloseKey(key);
   		log.info("RegQueryValueEx() [1] %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return defaultValue;
    }

    if ((result=RegQueryValueEx(key, valueName, 0,  NULL, &data, &size)) != ERROR_SUCCESS) {
        RegCloseKey(key);
   		log.info("RegQueryValueEx() [2] %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return defaultValue;
    }
    RegCloseKey(key);
    
    return data;
}

int TNCRegistryHelper::getRegBinary(HKEY preservedKey, const char *subKey, const char *valueName, BYTE **buffer) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "getRegBinary");

    HKEY key;
    DWORD size;
    long result;

    if ((result=RegOpenKeyEx(preservedKey, subKey, 0, KEY_READ, &key)) != ERROR_SUCCESS) {
   		log.info("RegOpenKeyEx() %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return 0;
    }
    
    if ((result=RegQueryValueEx(key, valueName, 0,  NULL, NULL, &size)) != ERROR_SUCCESS) {
        RegCloseKey(key);
   		log.info("RegQueryValueEx() [1] %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return 0;
    }
    
    *buffer = (BYTE*) calloc(sizeof(BYTE), size);

    if ((result=RegQueryValueEx(key, valueName, 0,  NULL, *buffer, &size)) != ERROR_SUCCESS) {
        RegCloseKey(key);
   		log.info("RegQueryValueEx() [2] %s \"%s\" %s fail.", getHKEYName(preservedKey), subKey, valueName);
    	log.info("Error: %d", result);
        return 0;
    }
    RegCloseKey(key);

    return size;
}

std::vector<std::string> * TNCRegistryHelper::getSubKeys(HKEY preservedKey, const char *subKey) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "getSubKeys");

    HKEY key;
    long result;

    if ((result=RegOpenKeyEx(preservedKey, subKey, 0, KEY_READ, &key)) != ERROR_SUCCESS) {
    	std::string s = std::string("RegOpenKeyEx() ") + getHKEYName(preservedKey) + std::string(" \"") + std::string(subKey) + std::string("\" fail.");
    	log.info("Error: %d", result);
    	log.info(s);
        return NULL;
    }
    
    std::vector<std::string> *subKeys = new std::vector<std::string>();
    int index = 0;
    DWORD size;
    char subKeyName[256];
    do {
        size = sizeof(subKeyName);
        if ((result = RegEnumKeyEx(key, index, subKeyName, &size, NULL, NULL, NULL, NULL)) == ERROR_SUCCESS) {
        	std::string foundSubKey(subKeyName);
            subKeys -> push_back(foundSubKey);
        } else {
    		log.info("RegEnumKeyEx() [%d] %s \"%s\" fail.", index, getHKEYName(preservedKey), subKey);
        }
        index++;
    } while (result != ERROR_NO_MORE_ITEMS);
    RegCloseKey(key);
  
    return subKeys;    
}

const char * TNCRegistryHelper::getHKEYName(const HKEY key) {
	if (key == HKEY_CLASSES_ROOT)
		return "HKEY_CLASSES_ROOT";
		
	if (key == HKEY_CURRENT_USER)
		return "HKEY_CURRENT_USER";
		
	if (key == HKEY_LOCAL_MACHINE)
		return "HKEY_LOCAL_MACHINE";
		
	if (key == HKEY_USERS)
		return "HKEY_USERS";
		
	if (key == HKEY_PERFORMANCE_DATA)
		return "HKEY_PERFORMANCE_DATA";
		
	if (key == HKEY_CURRENT_CONFIG)
		return "HKEY_CURRENT_CONFIG";
		
	if (key == HKEY_DYN_DATA)
		return "HKEY_DYN_DATA";
		
	return "UNKNOWN_HKEY";
}

#endif // __WINDOWS__
