#ifdef __WINDOWS__
/*
 * This software is Copyright (C) 2006 FH Hannover
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to share the
 * source code of your application with everyone you distribute it to, and you
 * also want to give them the right to share who uses it. If you wish to use
 * TNC@FHH under Open Source Licensing, you must contribute all your source code
 * to the open source community in accordance with the GPL Version 2 when your
 * application is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are creating
 * proprietary applications and you are not prepared to distribute and share the
 * source code of your application. Contact tnc@inform.fh-hannover.de for details.
 */

/**
 * WMIAccessHelper implementation. Provides helper methods to access WMI.
 *
 * @author: Daniel Wuttke
 * @date: 12/07/2006
 */
#include "WMIAccessHelper.h"

#define WMI_ACCESS_DLL "WMIAccess"
#define CLASSNAME "WMIAccessHelper"

// Function pointers
typedef bool (*OpenWMINamespacePointer)(BSTR wmiNamespace);
typedef char ** (*ExecuteWMIStatement)(BSTR statement, BSTR values[], int valueCount, int skip);
typedef void (*CloseWMINamespace)();

static HMODULE module = NULL;
static bool namespaceOpen = false;


FARPROC getProcAddress(const char *name) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "getProcAddress");

    log.debug("Try to find function %s.", name);
    FARPROC proc = GetProcAddress(module, name);
    if (proc == NULL) {
        log.error("Function %s was not found!", name);
    }
    return proc;
}

bool WMIAccessHelper::openWMINamespace(BSTR wmiNamespace, const char *pathToDLL) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "openWMINamespace");

    std::string dll;
    if (module == NULL) {
        dll = pathToDLL;
        dll.append("\\");
        dll.append(WMI_ACCESS_DLL);
        module = LoadLibrary(dll.c_str());
        log.debug("dll: %s", dll.c_str());
    }
    if (module != NULL) {
        log.debug("DLL %s was found.", WMI_ACCESS_DLL);

        OpenWMINamespacePointer proc = (OpenWMINamespacePointer) getProcAddress("openWMINamespace");
        if (proc != NULL) {
            namespaceOpen = proc(wmiNamespace);
        }
    } else {
        log.error("DLL %s was not found: %d", dll.c_str(), GetLastError());
    }
    log.debug("Namespace open? %d", namespaceOpen);
    return namespaceOpen;
}

char ** WMIAccessHelper::executeWMIStatement(BSTR statement, BSTR values[], int valueCount, int skip) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "executeWMIStatement");
    char **ret = NULL;

    if (module != NULL && namespaceOpen) {
        ExecuteWMIStatement proc = (ExecuteWMIStatement) getProcAddress("executeWMIStatement");
        if (proc != NULL) {
             ret = proc(statement, values, valueCount, skip);
             if (ret != NULL && log.isDebug()) {
                 for (int i = 0; i < valueCount; i++) {
                    log.debug("Result %d: %s", i+1, ret[i]);
                 }
             } else {
                log.warn("Nothing found!");
             }
        }
    } else {
        log.error("Invoke openWMINamespace() first!");
    }
    return ret;
}


void WMIAccessHelper::closeWMINamespace() {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "closeWMINamespace");

    if (module != NULL && namespaceOpen) {
        CloseWMINamespace proc = (CloseWMINamespace) getProcAddress("closeWMINamespace");
        if (proc != NULL) {
            proc();
            log.debug("Namespace closed.");
        }
        //log.debug("Free library.");
        //FreeLibrary(module);
        //module = NULL;
        namespaceOpen = false;
    } else {
        log.error("Invoke openWMINamespace() first!");
    }
    log.debug("Done.");
}

std::string WMIAccessHelper::isServiceRunning(BSTR serviceName, const char *pathToDLL) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "isServiceRunning");
    std::string ret = "false";

    if (WMIAccessHelper::openWMINamespace(L"root\\CIMV2", pathToDLL)) {
        wcslen(serviceName);
        BSTR stmtPart1 = L"Select * from Win32_Service where Name = \"";
        BSTR stmtPart2 = serviceName;
        BSTR stmtPart3 = L"\"";

        short length1 = wcslen(stmtPart1);
        short length2 = wcslen(serviceName);
        short length3 = wcslen(stmtPart3);
        short pos1 = 0;
        short pos2 = pos1 + length1;
        short pos3 = pos2 + length2;

        short stmtLength = length1 + length2 + length3 + 1;
        BSTR stmt = (BSTR) calloc(sizeof(OLECHAR), stmtLength);

        memcpy(stmt + pos1, stmtPart1, length1 * 2);
        memcpy(stmt + pos2, stmtPart2, length2 * 2);
        memcpy(stmt + pos3, stmtPart3, length3 * 2);

        /*
        printf("%d, %d, %d\n", length1, length2, length3);
        for (int i = 0; i < length1; i++) {
            printf("%x ", stmtPart1[i]);
        }
        cout << endl;
        for (int i = 0; i < stmtLength; i++) {
            printf("%c", stmt[i]);
        }
        cout << endl;
        log.debug("Statement: %s", TNCUtil::convertWCharString(stmt).c_str());
        */

        BSTR values[1];
        values[0] = L"Started";
        char **results = WMIAccessHelper::executeWMIStatement(stmt, values, 1);
        if (results != NULL) {
            if (!strcmp(results[0], "-1")) {
                ret = "true";
            } else if (!strcmp(results[0], "0")) {
                ret = "false";
            } else {
                ret = results[0];
            }
            log.debug("Result: %s", ret.c_str());
        }
        free(stmt);
        WMIAccessHelper::closeWMINamespace();
    } else {
        log.error("Cannot access WMI namespace root\\CIMV2!");
    }
    return ret;
}

std::string WMIAccessHelper::isProcessRunning(BSTR processName, const char *pathToDLL) {
    TNCLog &log = TNCLog::getLog(CLASSNAME, "isProcessRunning");
    std::string ret = "false";

    if (WMIAccessHelper::openWMINamespace(L"root\\CIMV2", pathToDLL)) {
        wcslen(processName);
        BSTR stmtPart1 = L"Select * from Win32_Process where Name = \"";
        BSTR stmtPart2 = processName;
        BSTR stmtPart3 = L"\"";

        short length1 = wcslen(stmtPart1);
        short length2 = wcslen(processName);
        short length3 = wcslen(stmtPart3);
        short pos1 = 0;
        short pos2 = pos1 + length1;
        short pos3 = pos2 + length2;

        short stmtLength = length1 + length2 + length3 + 1;
        BSTR stmt = (BSTR) calloc(sizeof(OLECHAR), stmtLength);

        memcpy(stmt + pos1, stmtPart1, length1 * 2);
        memcpy(stmt + pos2, stmtPart2, length2 * 2);
        memcpy(stmt + pos3, stmtPart3, length3 * 2);

        BSTR values[1];
        values[0] = L"Name";
        char **results = WMIAccessHelper::executeWMIStatement(stmt, values, 1);
        if (results != NULL) {
            ret = "true";
            log.debug("Result: %s", ret.c_str());
        }
        free(stmt);
        WMIAccessHelper::closeWMINamespace();
    } else {
        log.error("Cannot access WMI namespace root\\CIMV2!");
    }
    return ret;
}
#endif /*__WINDOWS__*/
